#define MOZ_SOURCE_STAMP 0f339c1e154f75c484fe2fac507a9a225990d212
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/0f339c1e154f75c484fe2fac507a9a225990d212
